@@ -1,25 +1,21 @@
+{-# LANGUAGE
+    RankNTypes
+  , FlexibleInstances
+  , MultiParamTypeClasses
+  , UndecidableInstances
+  , Trustworthy
+  #-}
+
 {-| This module is the recommended entry point to the @pipes@ library.
 
     Read "Pipes.Tutorial" if you want a tutorial explaining how to use this
     library.
 -}
 
-{-# LANGUAGE
-    RankNTypes
-  , CPP
-  , FlexibleInstances
-  , MultiParamTypeClasses
-  , UndecidableInstances
-  #-}
-
--- The rewrite RULES require the 'TrustWorthy' annotation
-#if __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Trustworthy #-}
-#endif
-
 module Pipes (
     -- * The Proxy Monad Transformer
       Proxy
+    , X
     , Effect
     , Effect'
     , runEffect
@@ -62,38 +58,29 @@ module Pipes (
     -- $reexports
     , module Control.Monad.IO.Class
     , module Control.Monad.Trans.Class
-#ifndef haskell98
     , module Control.Monad.Morph
-#endif
     , module Data.Foldable
-    , module Data.Void
     ) where
 
 import Control.Applicative (Applicative(pure, (<*>)), Alternative(empty, (<|>)))
-import Control.Monad (MonadPlus(mzero, mplus))
-import Control.Monad.IO.Class (MonadIO(liftIO)) -- transformers
-import Control.Monad.Trans.Class (MonadTrans(lift)) --transformers
-import Control.Monad.Trans.Error (ErrorT(runErrorT))
-import Control.Monad.Trans.Identity (IdentityT(runIdentityT)) --transformers
-import Control.Monad.Trans.Maybe (MaybeT(runMaybeT)) --transformers
-import Data.Foldable (Foldable)
-import qualified Data.Foldable as F
-import Data.Monoid (Monoid(..))
-import Data.Void (Void)
-import qualified Data.Void as V
-import Pipes.Internal (Proxy(..))
-import Pipes.Core
-#ifndef haskell98
 import Control.Monad.Error (MonadError(..))
+import Control.Monad.IO.Class (MonadIO(liftIO))
+import Control.Monad (MonadPlus(mzero, mplus))
 import Control.Monad.Reader (MonadReader(..))
 import Control.Monad.State (MonadState(..))
+import Control.Monad.Trans.Class (MonadTrans(lift))
+import Control.Monad.Trans.Error (ErrorT(runErrorT))
+import Control.Monad.Trans.Identity (IdentityT(runIdentityT))
+import Control.Monad.Trans.Maybe (MaybeT(runMaybeT))
 import Control.Monad.Writer (MonadWriter(..))
-#endif
+import Data.Foldable (Foldable)
+import Data.Monoid (Monoid(..))
+import Pipes.Core
+import Pipes.Internal (Proxy(..))
+import qualified Data.Foldable as F
 
 -- Re-exports
-#ifndef haskell98
 import Control.Monad.Morph (MFunctor(hoist))
-#endif
 
 infixl 4 <~
 infixr 4 ~>
@@ -140,7 +127,7 @@ f '~>' 'yield' = f
 'yield' :: 'Monad' m => a -> 'Pipe' x a m ()
 @
 -}
-yield :: (Monad m) => a -> Producer' a m ()
+yield :: Monad m => a -> Producer' a m ()
 yield = respond
 {-# INLINABLE yield #-}
 
@@ -153,7 +140,7 @@ yield = respond
 'for' :: 'Monad' m => 'Pipe'   x b m r -> (b -> 'Pipe'     x c m ()) -> 'Pipe'     x c m r
 @
 -}
-for :: (Monad m)
+for :: Monad m
     =>       Proxy x' x b' b m a'
     -- ^
     -> (b -> Proxy x' x c' c m b')
@@ -190,6 +177,14 @@ for = (//>)
                     yield x
                     go
             in  go
+
+  ; "p1 >-> (p2 >-> p3)" forall p1 p2 p3 .
+        p1 >-> (p2 >-> p3) = (p1 >-> p2) >-> p3
+
+  ; "p >-> cat" forall p . p >-> cat = p
+
+  ; "cat >-> p" forall p . cat >-> p = p
+
   #-}
 
 {-| Compose loop bodies
@@ -202,7 +197,7 @@ for = (//>)
 @
 -}
 (~>)
-    :: (Monad m)
+    :: Monad m
     => (a -> Proxy x' x b' b m a')
     -- ^
     -> (b -> Proxy x' x c' c m b')
@@ -213,7 +208,7 @@ for = (//>)
 
 -- | ('~>') with the arguments flipped
 (<~)
-    :: (Monad m)
+    :: Monad m
     => (b -> Proxy x' x c' c m b')
     -- ^
     -> (a -> Proxy x' x b' b m a')
@@ -246,7 +241,7 @@ f '>~' 'await' = f
 'await' :: 'Monad' m => 'Pipe' a y m a
 @
 -}
-await :: (Monad m) => Consumer' a m a
+await :: Monad m => Consumer' a m a
 await = request ()
 {-# INLINABLE await #-}
 
@@ -260,7 +255,7 @@ await = request ()
 @
 -}
 (>~)
-    :: (Monad m)
+    :: Monad m
     => Proxy a' a y' y m b
     -- ^
     -> Proxy () b y' y m c
@@ -271,7 +266,7 @@ p1 >~ p2 = (\() -> p1) >\\ p2
 
 -- | ('>~') with the arguments flipped
 (~<)
-    :: (Monad m)
+    :: Monad m
     => Proxy () b y' y m c
     -- ^
     -> Proxy a' a y' y m b
@@ -299,7 +294,7 @@ f '>->' 'cat' = f
 -}
 
 -- | The identity 'Pipe', analogous to the Unix @cat@ program
-cat :: (Monad m) => Pipe a a m r
+cat :: Monad m => Pipe a a m r
 cat = pull ()
 {-# INLINABLE cat #-}
 
@@ -313,7 +308,7 @@ cat = pull ()
 @
 -}
 (>->)
-    :: (Monad m)
+    :: Monad m
     => Proxy a' a () b m r
     -- ^
     -> Proxy () b c' c m r
@@ -364,29 +359,22 @@ instance (Monad m) => MonadPlus (ListT m) where
     mzero = empty
     mplus = (<|>)
 
-#ifndef haskell98
 instance MFunctor ListT where
     hoist morph = Select . hoist morph . enumerate
-#endif
 
 instance (Monad m) => Monoid (ListT m a) where
     mempty = empty
     mappend = (<|>)
 
-#ifndef haskell98
 instance (MonadState s m) => MonadState s (ListT m) where
     get     = lift  get
 
     put   s = lift (put   s)
 
-#if MIN_VERSION_mtl(2,1,0)
     state f = lift (state f)
-#endif
 
 instance (MonadWriter w m) => MonadWriter w (ListT m) where
-#if MIN_VERSION_mtl(2,1,0)
     writer = lift . writer
-#endif
 
     tell w = lift (tell w)
 
@@ -409,28 +397,25 @@ instance (MonadWriter w m) => MonadWriter w (ListT m) where
             M               m   -> M (do
                 (p', w') <- listen m
                 return (go p' $! mappend w w') )
-            Pure    r           -> Pure r
+            Pure     r          -> Pure r
 
 instance (MonadReader i m) => MonadReader i (ListT m) where
     ask = lift ask
 
     local f l = Select (local f (enumerate l))
 
-#if MIN_VERSION_mtl(2,1,0)
     reader f = lift (reader f)
-#endif
 
 instance (MonadError e m) => MonadError e (ListT m) where
     throwError e = lift (throwError e)
 
     catchError l k = Select (catchError (enumerate l) (\e -> enumerate (k e)))
-#endif
 
 {-| 'Enumerable' generalizes 'Data.Foldable.Foldable', converting effectful
     containers to 'ListT's.
 -}
 class Enumerable t where
-    toListT :: (Monad m) => t m a -> ListT m a
+    toListT :: Monad m => t m a -> ListT m a
 
 instance Enumerable ListT where
     toListT = id
@@ -459,20 +444,27 @@ instance Enumerable (ErrorT e) where
     'next' either fails with a 'Left' if the 'Producer' terminates or succeeds
     with a 'Right' providing the next value and the remainder of the 'Producer'.
 -}
-next :: (Monad m) => Producer a m r -> m (Either r (a, Producer a m r))
+next :: Monad m => Producer a m r -> m (Either r (a, Producer a m r))
 next = go
   where
     go p = case p of
-        Request v _  -> V.absurd v
+        Request v _  -> closed v
         Respond a fu -> return (Right (a, fu ()))
         M         m  -> m >>= go
         Pure    r    -> return (Left r)
 {-# INLINABLE next #-}
 
 -- | Convert a 'F.Foldable' to a 'Producer'
-each :: (Monad m, F.Foldable f) => f a -> Producer' a m ()
-each = F.mapM_ yield
+each :: (Monad m, Foldable f) => f a -> Producer' a m ()
+each = F.foldr (\a p -> yield a >> p) (return ())
 {-# INLINABLE each #-}
+{-  The above code is the same as:
+
+> each = Data.Foldable.mapM_ yield
+
+    ... except writing it directly in terms of `Data.Foldable.foldr` improves
+    build/foldr fusion
+-}
 
 -- | Convert an 'Enumerable' to a 'Producer'
 every :: (Monad m, Enumerable t) => t m a -> Producer' a m ()
@@ -480,13 +472,13 @@ every it = discard >\\ enumerate (toListT it)
 {-# INLINABLE every #-}
 
 -- | Discards a value
-discard :: (Monad m) => a -> m ()
+discard :: Monad m => a -> m ()
 discard _ = return ()
 {-# INLINABLE discard #-}
 
 -- | ('>->') with the arguments flipped
 (<-<)
-    :: (Monad m)
+    :: Monad m
     => Proxy () b c' c m r
     -- ^
     -> Proxy a' a () b m r
@@ -500,11 +492,7 @@ p2 <-< p1 = p1 >-> p2
 
     "Control.Monad.Trans.Class" re-exports 'MonadTrans'.
 
-#ifndef haskell98
     "Control.Monad.Morph" re-exports 'MFunctor'.
 
-#endif
     "Data.Foldable" re-exports 'Foldable' (the class name only)
-
-    "Data.Void" re-exports 'Void'.
 -}
