@@ -13,12 +13,7 @@
     * push-based 'Pipe's.
 -}
 
-{-# LANGUAGE CPP, RankNTypes #-}
-
--- The rewrite RULES require the 'TrustWorthy' annotation
-#if __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Trustworthy #-}
-#endif
+{-# LANGUAGE RankNTypes, Trustworthy #-}
 
 module Pipes.Core (
     -- * Proxy Monad Transformer
@@ -58,6 +53,7 @@ module Pipes.Core (
     , reflect
 
     -- * Concrete Type Synonyms
+    , X
     , Effect
     , Producer
     , Pipe
@@ -81,10 +77,12 @@ module Pipes.Core (
     , (<\\)
     , (//<)
     , (<<+)
+
+    -- * Re-exports
+    , closed
     ) where
 
-import Data.Void (Void, absurd)
-import Pipes.Internal (Proxy(..))
+import Pipes.Internal (Proxy(..), X, closed)
 
 {- $proxy
     Diagrammatically, you can think of a 'Proxy' as having the following shape:
@@ -117,12 +115,12 @@ import Pipes.Internal (Proxy(..))
 -}
 
 -- | Run a self-contained 'Effect', converting it back to the base monad
-runEffect :: (Monad m) => Effect m r -> m r
+runEffect :: Monad m => Effect m r -> m r
 runEffect = go
   where
     go p = case p of
-        Request v _ -> absurd v
-        Respond v _ -> absurd v
+        Request v _ -> closed v
+        Respond v _ -> closed v
         M       m   -> m >>= go
         Pure    r   -> return r
 {-# INLINABLE runEffect #-}
@@ -221,7 +219,7 @@ f '/>/' 'respond' = f
     The following diagrams show the flow of information:
 
 @
-'respond' :: ('Monad' m)
+'respond' :: 'Monad' m
        =>  a -> 'Proxy' x' x a' a m a'
 
 \          a
@@ -236,7 +234,7 @@ f '/>/' 'respond' = f
           v 
           a'
 
-('/>/') :: ('Monad' m)
+('/>/') :: 'Monad' m
       => (a -> 'Proxy' x' x b' b m a')
       -> (b -> 'Proxy' x' x c' c m b')
       -> (a -> 'Proxy' x' x b' b m a')
@@ -261,7 +259,7 @@ f '/>/' 'respond' = f
 
     'respond' is the identity of the respond category.
 -}
-respond :: (Monad m) => a -> Proxy x' x a' a m a'
+respond :: Monad m => a -> Proxy x' x a' a m a'
 respond a = Respond a Pure
 {-# INLINABLE respond #-}
 
@@ -274,7 +272,7 @@ respond a = Respond a Pure
     ('/>/') is the composition operator of the respond category.
 -}
 (/>/)
-    :: (Monad m)
+    :: Monad m
     => (a -> Proxy x' x b' b m a')
     -- ^
     -> (b -> Proxy x' x c' c m b')
@@ -289,7 +287,7 @@ respond a = Respond a Pure
     Point-ful version of ('/>/')
 -}
 (//>)
-    :: (Monad m)
+    :: Monad m
     =>       Proxy x' x b' b m a'
     -- ^
     -> (b -> Proxy x' x c' c m b')
@@ -336,7 +334,7 @@ f '\>\' 'request' = f
     The following diagrams show the flow of information:
 
 @
-'request' :: ('Monad' m)
+'request' :: 'Monad' m
         =>  a' -> 'Proxy' a' a y' y m a
 
 \          a'
@@ -351,7 +349,7 @@ f '\>\' 'request' = f
           v
           a
 
-('\>\') :: ('Monad' m)
+('\>\') :: 'Monad' m
       => (b' -> 'Proxy' a' a y' y m b)
       -> (c' -> 'Proxy' b' b y' y m c)
       -> (c' -> 'Proxy' a' a y' y m c)
@@ -374,7 +372,7 @@ f '\>\' 'request' = f
 
     'request' is the identity of the request category.
 -}
-request :: (Monad m) => a' -> Proxy a' a y' y m a
+request :: Monad m => a' -> Proxy a' a y' y m a
 request a' = Request a' Pure
 {-# INLINABLE request #-}
 
@@ -387,7 +385,7 @@ request a' = Request a' Pure
     ('\>\') is the composition operator of the request category.
 -}
 (\>\)
-    :: (Monad m)
+    :: Monad m
     => (b' -> Proxy a' a y' y m b)
     -- ^
     -> (c' -> Proxy b' b y' y m c)
@@ -402,7 +400,7 @@ request a' = Request a' Pure
     Point-ful version of ('\>\')
 -}
 (>\\)
-    :: (Monad m)
+    :: Monad m
     => (b' -> Proxy a' a y' y m b)
     -- ^
     ->        Proxy b' b y' y m c
@@ -449,7 +447,7 @@ f '>~>' 'push' = f
     The following diagram shows the flow of information:
 
 @
-'push'  :: ('Monad' m)
+'push'  :: 'Monad' m
       =>  a -> 'Proxy' a' a a' a m r
 
 \          a
@@ -464,7 +462,7 @@ f '>~>' 'push' = f
           v
           r
 
-('>~>') :: ('Monad' m)
+('>~>') :: 'Monad' m
       => (a -> 'Proxy' a' a b' b m r)
       -> (b -> 'Proxy' b' b c' c m r)
       -> (a -> 'Proxy' a' a c' c m r)
@@ -492,7 +490,7 @@ f '>~>' 'push' = f
 
     'push' is the identity of the push category.
 -}
-push :: (Monad m) => a -> Proxy a' a a' a m r
+push :: Monad m => a -> Proxy a' a a' a m r
 push = go
   where
     go a = Respond a (\a' -> Request a' go)
@@ -508,7 +506,7 @@ push = go
     ('>~>') is the composition operator of the push category.
 -}
 (>~>)
-    :: (Monad m)
+    :: Monad m
     => (_a -> Proxy a' a b' b m r)
     -- ^
     -> ( b -> Proxy b' b c' c m r)
@@ -523,7 +521,7 @@ push = go
     Point-ful version of ('>~>')
 -}
 (>>~)
-    :: (Monad m)
+    :: Monad m
     =>       Proxy a' a b' b m r
     -- ^
     -> (b -> Proxy b' b c' c m r)
@@ -557,7 +555,7 @@ f '>+>' 'pull' = f
     The following diagrams show the flow of information:
 
 @
-'pull'  :: ('Monad' m)
+'pull'  :: 'Monad' m
       =>  a' -> 'Proxy' a' a a' a m r
 
 \          a'
@@ -572,7 +570,7 @@ f '>+>' 'pull' = f
           v
           r
 
-('>+>') :: ('Monad' m)
+('>+>') :: 'Monad' m
       -> (b' -> 'Proxy' a' a b' b m r)
       -> (c' -> 'Proxy' b' b c' c m r)
       -> (c' -> 'Proxy' a' a c' c m r)
@@ -600,7 +598,7 @@ f '>+>' 'pull' = f
 
     'pull' is the identity of the pull category.
 -}
-pull :: (Monad m) => a' -> Proxy a' a a' a m r
+pull :: Monad m => a' -> Proxy a' a a' a m r
 pull = go
   where
     go a' = Request a' (\a -> Respond a go)
@@ -616,7 +614,7 @@ pull = go
     ('>+>') is the composition operator of the pull category.
 -}
 (>+>)
-    :: (Monad m)
+    :: Monad m
     => ( b' -> Proxy a' a b' b m r)
     -- ^
     -> (_c' -> Proxy b' b c' c m r)
@@ -631,7 +629,7 @@ pull = go
     Point-ful version of ('>+>')
 -}
 (+>>)
-    :: (Monad m)
+    :: Monad m
     => (b' -> Proxy a' a b' b m r)
     -- ^
     ->        Proxy b' b c' c m r
@@ -678,7 +676,7 @@ fb' +>> p = case p of
 -}
 
 -- | Switch the upstream and downstream ends
-reflect :: (Monad m) => Proxy a' a b' b m r -> Proxy b b' a a' m r
+reflect :: Monad m => Proxy a' a b' b m r -> Proxy b b' a a' m r
 reflect = go
   where
     go p = case p of
@@ -692,30 +690,30 @@ reflect = go
 
     'Effect's neither 'Pipes.await' nor 'Pipes.yield'
 -}
-type Effect = Proxy Void () () Void
+type Effect = Proxy X () () X
 
 -- | 'Producer's can only 'Pipes.yield'
-type Producer b = Proxy Void () () b
+type Producer b = Proxy X () () b
 
 -- | 'Pipe's can both 'Pipes.await' and 'Pipes.yield'
 type Pipe a b = Proxy () a () b
 
 -- | 'Consumer's can only 'Pipes.await'
-type Consumer a = Proxy () a () Void
+type Consumer a = Proxy () a () X
 
 {-| @Client a' a@ sends requests of type @a'@ and receives responses of
     type @a@.
 
     'Client's only 'request' and never 'respond'.
 -}
-type Client a' a = Proxy a' a () Void
+type Client a' a = Proxy a' a () X
 
 {-| @Server b' b@ receives requests of type @b'@ and sends responses of type
     @b@.
 
     'Server's only 'respond' and never 'request'.
 -}
-type Server b' b = Proxy Void () b' b
+type Server b' b = Proxy X () b' b
 
 -- | Like 'Effect', but with a polymorphic type
 type Effect' m r = forall x' x y' y . Proxy x' x y' y m r
@@ -734,7 +732,7 @@ type Client' a' a m r = forall y' y . Proxy a' a y' y m r
 
 -- | Equivalent to ('/>/') with the arguments flipped
 (\<\)
-    :: (Monad m)
+    :: Monad m
     => (b -> Proxy x' x c' c m b')
     -- ^
     -> (a -> Proxy x' x b' b m a')
@@ -746,7 +744,7 @@ p1 \<\ p2 = p2 />/ p1
 
 -- | Equivalent to ('\>\') with the arguments flipped
 (/</)
-    :: (Monad m)
+    :: Monad m
     => (c' -> Proxy b' b x' x m c)
     -- ^
     -> (b' -> Proxy a' a x' x m b)
@@ -758,7 +756,7 @@ p1 /</ p2 = p2 \>\ p1
 
 -- | Equivalent to ('>~>') with the arguments flipped
 (<~<)
-    :: (Monad m)
+    :: Monad m
     => (b -> Proxy b' b c' c m r)
     -- ^
     -> (a -> Proxy a' a b' b m r)
@@ -770,7 +768,7 @@ p1 <~< p2 = p2 >~> p1
 
 -- | Equivalent to ('>+>') with the arguments flipped
 (<+<)
-    :: (Monad m)
+    :: Monad m
     => (c' -> Proxy b' b c' c m r)
     -- ^
     -> (b' -> Proxy a' a b' b m r)
@@ -782,7 +780,7 @@ p1 <+< p2 = p2 >+> p1
 
 -- | Equivalent to ('//>') with the arguments flipped
 (<\\)
-    :: (Monad m)
+    :: Monad m
     => (b -> Proxy x' x c' c m b')
     -- ^
     ->       Proxy x' x b' b m a'
@@ -794,7 +792,7 @@ f <\\ p = p //> f
 
 -- | Equivalent to ('>\\') with the arguments flipped
 (//<)
-    :: (Monad m)
+    :: Monad m
     =>        Proxy b' b y' y m c
     -- ^
     -> (b' -> Proxy a' a y' y m b)
@@ -806,7 +804,7 @@ p //< f = f >\\ p
 
 -- | Equivalent to ('>>~') with the arguments flipped
 (~<<)
-    :: (Monad m)
+    :: Monad m
     => (b  -> Proxy b' b c' c m r)
     -- ^
     ->        Proxy a' a b' b m r
@@ -818,7 +816,7 @@ k ~<< p = p >>~ k
 
 -- | Equivalent to ('+>>') with the arguments flipped
 (<<+)
-    :: (Monad m)
+    :: Monad m
     =>         Proxy b' b c' c m r
     -- ^
     -> (b'  -> Proxy a' a b' b m r)
@@ -829,16 +827,28 @@ k <<+ p = p +>> k
 {-# INLINABLE (<<+) #-}
 
 {-# RULES
-    "(p //> f) //> g" forall p f g . (p //> f) //> g = p //> (\a -> f a //> g)
+    "(p //> f) //> g" forall p f g . (p //> f) //> g = p //> (\x -> f x //> g)
 
   ; "p //> respond" forall p . p //> respond = p
 
   ; "respond x //> f" forall x f . respond x //>  f = f x
 
-  ; "f >\\ (g >\\ p)" forall f g p . f >\\ (g >\\ p) = (\a -> f >\\ g a) >\\ p
+  ; "f >\\ (g >\\ p)" forall f g p . f >\\ (g >\\ p) = (\x -> f >\\ g x) >\\ p
 
   ; "request >\\ p" forall p . request >\\ p = p
 
   ; "f >\\ request x" forall f x . f >\\ request x = f x
 
+  ; "(p >>~ f) >>~ g" forall p f g . (p >>~ f) >>~ g = p >>~ (\x -> f x >>~ g)
+
+  ; "p >>~ push" forall p . p >>~ push = p
+
+  ; "push x >>~ f" forall x f . push x >>~ f = f x
+
+  ; "f +>> (g +>> p)" forall f g p . f +>> (g +>> p) = (\x -> f +>> g x) +>> p
+
+  ; "pull +>> p" forall p . pull +>> p = p
+
+  ; "f +>> pull x" forall f x . f +>> pull x = f x
+
   #-}
