@@ -23,33 +23,28 @@
   , MultiParamTypeClasses
   , RankNTypes
   , UndecidableInstances
-  , CPP
+  , Trustworthy
   #-}
 
--- The rewrite RULES require the 'TrustWorthy' annotation
-#if __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Trustworthy #-}
-#endif
-
 module Pipes.Internal (
     -- * Internal
       Proxy(..)
     , unsafeHoist
-    , observe,
+    , observe
+    , X
+    , closed
     ) where
 
 import Control.Applicative (Applicative(pure, (<*>)), Alternative(empty, (<|>)))
 import Control.Monad (MonadPlus(..))
 import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad.Trans.Class (MonadTrans(lift))
-#ifndef haskell98
 import Control.Monad.Morph (MFunctor(hoist))
 import Control.Monad.Error (MonadError(..))
 import Control.Monad.Reader (MonadReader(..))
 import Control.Monad.State (MonadState(..))
 import Control.Monad.Writer (MonadWriter(..))
 import Data.Monoid (mempty,mappend)
-#endif
 
 {-| A 'Proxy' is a monad transformer that receives and sends information on both
     an upstream and downstream interface.
@@ -72,7 +67,7 @@ data Proxy a' a b' b m r
     | M          (m    (Proxy a' a b' b m r))
     | Pure    r
 
-instance (Monad m) => Functor (Proxy a' a b' b m) where
+instance Monad m => Functor (Proxy a' a b' b m) where
     fmap f p0 = go p0 where
         go p = case p of
             Request a' fa  -> Request a' (\a  -> go (fa  a ))
@@ -80,21 +75,21 @@ instance (Monad m) => Functor (Proxy a' a b' b m) where
             M          m   -> M (m >>= \p' -> return (go p'))
             Pure    r      -> Pure (f r)
 
-instance (Monad m) => Applicative (Proxy a' a b' b m) where
+instance Monad m => Applicative (Proxy a' a b' b m) where
     pure      = Pure
     pf <*> px = go pf where
         go p = case p of
             Request a' fa  -> Request a' (\a  -> go (fa  a ))
             Respond b  fb' -> Respond b  (\b' -> go (fb' b'))
             M          m   -> M (m >>= \p' -> return (go p'))
-            Pure     f     -> fmap f px
+            Pure    f      -> fmap f px
 
-instance (Monad m) => Monad (Proxy a' a b' b m) where
+instance Monad m => Monad (Proxy a' a b' b m) where
     return = Pure
     (>>=)  = _bind
 
 _bind
-    :: (Monad m)
+    :: Monad m
     => Proxy a' a b' b m r
     -> (r -> Proxy a' a b' b m r')
     -> Proxy a' a b' b m r'
@@ -103,7 +98,7 @@ p0 `_bind` f = go p0 where
         Request a' fa  -> Request a' (\a  -> go (fa  a ))
         Respond b  fb' -> Respond b  (\b' -> go (fb' b'))
         M          m   -> M (m >>= \p' -> return (go p'))
-        Pure     r     -> f r
+        Pure    r      -> f r
 
 {-# RULES
     "_bind (Request a' k) f" forall a' k f .
@@ -126,7 +121,7 @@ instance MonadTrans (Proxy a' a b' b) where
     safe if you pass a monad morphism as the first argument.
 -}
 unsafeHoist
-    :: (Monad m)
+    :: Monad m
     => (forall x . m x -> n x) -> Proxy a' a b' b m r -> Proxy a' a b' b n r
 unsafeHoist nat = go
   where
@@ -134,24 +129,21 @@ unsafeHoist nat = go
         Request a' fa  -> Request a' (\a  -> go (fa  a ))
         Respond b  fb' -> Respond b  (\b' -> go (fb' b'))
         M          m   -> M (nat (m >>= \p' -> return (go p')))
-        Pure       r   -> Pure r
+        Pure    r      -> Pure r
 {-# INLINABLE unsafeHoist #-}
 
-#ifndef haskell98
 instance MFunctor (Proxy a' a b' b) where
     hoist nat p0 = go (observe p0) where
         go p = case p of
             Request a' fa  -> Request a' (\a  -> go (fa  a ))
             Respond b  fb' -> Respond b  (\b' -> go (fb' b'))
             M          m   -> M (nat (m >>= \p' -> return (go p')))
-            Pure       r   -> Pure r
-#endif
+            Pure    r      -> Pure r
 
-instance (MonadIO m) => MonadIO (Proxy a' a b' b m) where
+instance MonadIO m => MonadIO (Proxy a' a b' b m) where
     liftIO m = M (liftIO (m >>= \r -> return (Pure r)))
 
-#ifndef haskell98
-instance (MonadReader r m) => MonadReader r (Proxy a' a b' b m) where
+instance MonadReader r m => MonadReader r (Proxy a' a b' b m) where
     ask = lift ask
     local f = go
         where
@@ -160,21 +152,15 @@ instance (MonadReader r m) => MonadReader r (Proxy a' a b' b m) where
               Respond b  fb' -> Respond b  (\b' -> go (fb' b'))
               Pure    r      -> Pure r
               M       m      -> M (local f m >>= \r -> return (go r))
-#if MIN_VERSION_mtl(2,1,0)
     reader = lift . reader
-#endif
 
-instance (MonadState s m) => MonadState s (Proxy a' a b' b m) where
+instance MonadState s m => MonadState s (Proxy a' a b' b m) where
     get = lift get
     put = lift . put
-#if MIN_VERSION_mtl(2,1,0)
     state = lift . state
-#endif
 
-instance (MonadWriter w m) => MonadWriter w (Proxy a' a b' b m) where
-#if MIN_VERSION_mtl(2,1,0)
+instance MonadWriter w m => MonadWriter w (Proxy a' a b' b m) where
     writer = lift . writer
-#endif
     tell = lift . tell
     listen p0 = go p0 mempty
       where
@@ -194,9 +180,9 @@ instance (MonadWriter w m) => MonadWriter w (Proxy a' a b' b m) where
             M       m      -> M (do
                 (p', w') <- listen m
                 return (go p' $! mappend w w') )
-            Pure    (r, f) -> M (pass (return (Pure r, \_ -> f w)))
+            Pure   (r, f)  -> M (pass (return (Pure r, \_ -> f w)))
 
-instance (MonadError e m) => MonadError e (Proxy a' a b' b m) where
+instance MonadError e m => MonadError e (Proxy a' a b' b m) where
     throwError = lift . throwError
     catchError p0 f = go p0
       where
@@ -207,13 +193,12 @@ instance (MonadError e m) => MonadError e (Proxy a' a b' b m) where
             M          m   -> M ((do
                 p' <- m
                 return (go p') ) `catchError` (\e -> return (f e)) )
-#endif
 
-instance (MonadPlus m) => Alternative (Proxy a' a b' b m) where
+instance MonadPlus m => Alternative (Proxy a' a b' b m) where
     empty = mzero
     (<|>) = mplus
 
-instance (MonadPlus m) => MonadPlus (Proxy a' a b' b m) where
+instance MonadPlus m => MonadPlus (Proxy a' a b' b m) where
     mzero = lift mzero
     mplus p0 p1 = go p0
       where
@@ -240,7 +225,7 @@ instance (MonadPlus m) => MonadPlus (Proxy a' a b' b m) where
     This function is a convenience for low-level @pipes@ implementers.  You do
     not need to use 'observe' if you stick to the safe API.
 -}
-observe :: (Monad m) => Proxy a' a b' b m r -> Proxy a' a b' b m r
+observe :: Monad m => Proxy a' a b' b m r -> Proxy a' a b' b m r
 observe p0 = M (go p0) where
     go p = case p of
         Request a' fa  -> return (Request a' (\a  -> observe (fa  a )))
@@ -248,3 +233,16 @@ observe p0 = M (go p0) where
         M          m'  -> m' >>= go
         Pure    r      -> return (Pure r)
 {-# INLINABLE observe #-}
+
+{-| The empty type, used to close output ends
+
+    When @Data.Void@ is merged into @base@, this will change to:
+
+> type X = Void
+-}
+newtype X = X X
+
+-- | Use 'closed' to \"handle\" impossible outputs
+closed :: X -> a
+closed (X x) = closed x
+{-# INLINABLE closed #-}
