@@ -1,58 +1,49 @@
 {-| Many actions in base monad transformers cannot be automatically
     'Control.Monad.Trans.Class.lift'ed.  These functions lift these remaining
     actions so that they work in the 'Proxy' monad transformer.
+
+    See the mini-tutorial at the bottom of this module for example code and
+    typical use cases where this module will come in handy.
 -}
 
-{-# LANGUAGE CPP #-}
-
 module Pipes.Lift (
+    -- * Utilities
+      distribute
+
     -- * ErrorT
-      errorP
-#ifndef haskell98
+    , errorP
     , runErrorP
     , catchError
-#endif
     , liftCatchError
 
     -- * MaybeT
     , maybeP
-#ifndef haskell98
     , runMaybeP
-#endif
 
     -- * ReaderT
     , readerP
-#ifndef haskell98
     , runReaderP
-#endif
 
     -- * StateT
     , stateP
-#ifndef haskell98
     , runStateP
     , evalStateP
     , execStateP
-#endif
 
     -- * WriterT
     -- $writert
     , writerP
-#ifndef haskell98
     , runWriterP
     , execWriterP
-#endif
 
     -- * RWST
     , rwsP
-#ifndef haskell98
     , runRWSP
     , evalRWSP
     , execRWSP
 
-    -- * Utilities
-    , distribute
-#endif
-
+    -- * Tutorial
+    -- $tutorial
     ) where
 
 import Control.Monad.Trans.Class (lift, MonadTrans(..))
@@ -64,10 +55,26 @@ import qualified Control.Monad.Trans.Writer.Strict as W
 import qualified Control.Monad.Trans.RWS.Strict as RWS
 import Data.Monoid (Monoid)
 import Pipes.Internal (Proxy(..), unsafeHoist)
-#ifndef haskell98
 import Control.Monad.Morph (hoist, MFunctor(..))
 import Pipes.Core (runEffect, request, respond, (//>), (>\\))
-#endif
+
+-- | Distribute 'Proxy' over a monad transformer
+distribute
+    ::  ( Monad m
+        , MonadTrans t
+        , MFunctor t
+        , Monad (t m)
+        , Monad (t (Proxy a' a b' b m))
+        )
+    => Proxy a' a b' b (t m) r
+    -- ^ 
+    -> t (Proxy a' a b' b m) r
+    -- ^ 
+distribute p =  runEffect $ request' >\\ unsafeHoist (hoist lift) p //> respond'
+  where
+    request' = lift . lift . request
+    respond' = lift . lift . respond
+{-# INLINABLE distribute #-}
 
 -- | Wrap the base monad in 'E.ErrorT'
 errorP
@@ -79,7 +86,6 @@ errorP p = do
     lift $ E.ErrorT (return x)
 {-# INLINABLE errorP #-}
 
-#ifndef haskell98
 -- | Run 'E.ErrorT' in the base monad
 runErrorP
     :: (Monad m, E.Error e)
@@ -99,11 +105,10 @@ catchError
 catchError e h = errorP . E.runErrorT $ 
     E.catchError (distribute e) (distribute . h)
 {-# INLINABLE catchError #-}
-#endif
 
 -- | Catch an error using a catch function for the base monad
 liftCatchError
-    :: (Monad m)
+    :: Monad m
     => (   m (Proxy a' a b' b m r)
         -> (e -> m (Proxy a' a b' b m r))
         -> m (Proxy a' a b' b m r) )
@@ -125,46 +130,42 @@ liftCatchError c p0 f = go p0
 
 -- | Wrap the base monad in 'M.MaybeT'
 maybeP
-    :: (Monad m)
+    :: Monad m
     => Proxy a' a b' b m (Maybe r) -> Proxy a' a b' b (M.MaybeT m) r
 maybeP p = do
     x <- unsafeHoist lift p
     lift $ M.MaybeT (return x)
 {-# INLINABLE maybeP #-}
 
-#ifndef haskell98
 -- | Run 'M.MaybeT' in the base monad
 runMaybeP
-    :: (Monad m)
+    :: Monad m
     => Proxy a' a b' b (M.MaybeT m) r
     -> Proxy a' a b' b m (Maybe r)
 runMaybeP p = M.runMaybeT $ distribute p
 {-# INLINABLE runMaybeP #-}
-#endif
 
 -- | Wrap the base monad in 'R.ReaderT'
 readerP
-    :: (Monad m)
+    :: Monad m
     => (i -> Proxy a' a b' b m r) -> Proxy a' a b' b (R.ReaderT i m) r
 readerP k = do
     i <- lift R.ask
     unsafeHoist lift (k i)
 {-# INLINABLE readerP #-}
 
-#ifndef haskell98
 -- | Run 'R.ReaderT' in the base monad
 runReaderP
-    :: (Monad m)
+    :: Monad m
     => i
     -> Proxy a' a b' b (R.ReaderT i m) r
     -> Proxy a' a b' b m r
 runReaderP r p = (`R.runReaderT` r) $ distribute p
 {-# INLINABLE runReaderP #-}
-#endif
 
 -- | Wrap the base monad in 'S.StateT'
 stateP
-    :: (Monad m)
+    :: Monad m
     => (s -> Proxy a' a b' b m (r, s)) -> Proxy a' a b' b (S.StateT s m) r
 stateP k = do
     s <- lift S.get
@@ -173,10 +174,9 @@ stateP k = do
     return r
 {-# INLINABLE stateP #-}
 
-#ifndef haskell98
 -- | Run 'S.StateT' in the base monad
 runStateP
-    :: (Monad m)
+    :: Monad m
     => s
     -> Proxy a' a b' b (S.StateT s m) r
     -> Proxy a' a b' b m (r, s)
@@ -185,7 +185,7 @@ runStateP s p = (`S.runStateT` s) $ distribute p
 
 -- | Evaluate 'S.StateT' in the base monad
 evalStateP
-    :: (Monad m)
+    :: Monad m
     => s
     -> Proxy a' a b' b (S.StateT s m) r
     -> Proxy a' a b' b m r
@@ -194,13 +194,12 @@ evalStateP s p = fmap fst $ runStateP s p
 
 -- | Execute 'S.StateT' in the base monad
 execStateP
-    :: (Monad m)
+    :: Monad m
     => s
     -> Proxy a' a b' b (S.StateT s m) r
     -> Proxy a' a b' b m s
 execStateP s p = fmap snd $ runStateP s p
 {-# INLINABLE execStateP #-}
-#endif
 
 {- $writert
     Note that 'runWriterP' and 'execWriterP' will keep the accumulator in
@@ -223,10 +222,9 @@ writerP p = do
     return r
 {-# INLINABLE writerP #-}
 
-#ifndef haskell98
 -- | Run 'W.WriterT' in the base monad
 runWriterP
-    :: (Monad m, Data.Monoid.Monoid w)
+    :: (Monad m, Monoid w)
     => Proxy a' a b' b (W.WriterT w m) r
     -> Proxy a' a b' b m (r, w)
 runWriterP p = W.runWriterT $ distribute p
@@ -234,12 +232,11 @@ runWriterP p = W.runWriterT $ distribute p
 
 -- | Execute 'W.WriterT' in the base monad
 execWriterP
-    :: (Monad m, Data.Monoid.Monoid w)
+    :: (Monad m, Monoid w)
     => Proxy a' a b' b (W.WriterT w m) r
     -> Proxy a' a b' b m w
 execWriterP p = fmap snd $ runWriterP p
 {-# INLINABLE execWriterP #-}
-#endif
 
 -- | Wrap the base monad in 'RWS.RWST'
 rwsP
@@ -256,7 +253,6 @@ rwsP k = do
     return r
 {-# INLINABLE rwsP #-}
 
-#ifndef haskell98
 -- | Run 'RWS.RWST' in the base monad
 runRWSP
     :: (Monad m, Monoid w)
@@ -291,21 +287,94 @@ execRWSP i s p = fmap f $ runRWSP i s p
     f x = let (_, s', w) = x in (s', w)
 {-# INLINABLE execRWSP #-}
 
--- | Distribute 'Proxy' over a monad transformer
-distribute
-    ::  ( Monad m
-        , MonadTrans t
-        , MFunctor t
-        , Monad (t m)
-        , Monad (t (Proxy a' a b' b m))
-        )
-    => Proxy a' a b' b (t m) r
-    -- ^ 
-    -> t (Proxy a' a b' b m) r
-    -- ^ 
-distribute p =  runEffect $ request' >\\ unsafeHoist (hoist lift) p //> respond'
-  where
-    request' = lift . lift . request
-    respond' = lift . lift . respond
-{-# INLINABLE distribute #-}
-#endif
+{- $tutorial
+    Probably the most useful functionality in this module is lifted error
+    handling.  Suppose that you have a 'Pipes.Pipe' whose base monad can fail
+    using 'E.ErrorT':
+
+> import Control.Monad.Trans.Error
+> import Pipes
+>
+> example :: Monad m => Pipe Int Int (ErrorT String m) r
+> example = for cat $ \n ->
+>     if n == 0
+>     then lift $ throwError "Zero is forbidden"
+>     else yield n
+
+    Without the tools in this module you cannot recover from any potential error
+    until after you compose and run the pipeline:
+
+>>> import qualified Pipes.Prelude as P
+>>> runErrorT $ runEffect $ P.readLn >-> example >-> P.print
+42<Enter>
+42
+1<Enter>
+1
+0<Enter>
+Zero is forbidden
+>>>
+
+    This module provides `catchError`, which lets you catch and recover from
+    errors inside the 'Pipe':
+
+>  import qualified Pipes.Lift as Lift
+> 
+>  caught :: Pipe Int Int (ErrorT String IO) r
+>  caught = example `Lift.catchError` \str -> do
+>      liftIO (putStrLn str)
+>      caught
+
+    This lets you resume streaming in the face of errors raised within the base
+    monad:
+
+>>> runErrorT $ runEffect $ P.readLn >-> caught >-> P.print
+0<Enter>
+Zero is forbidden
+42<Enter>
+42
+0<Enter>
+Zero is forbidden
+1<Enter>
+1
+...
+
+    Another common use case is running a base monad before running the pipeline.
+    For example, the following contrived 'Producer' uses 'S.StateT' gratuitously
+    to increment numbers:
+
+> import Control.Monad (forever)
+> import Control.Monad.Trans.State.Strict
+> import Pipes
+> 
+> numbers :: Monad m => Producer Int (StateT Int m) r
+> numbers = forever $ do
+>     n <- lift get
+>     yield n
+>     lift $ put $! n + 1
+
+    You can run the 'StateT' monad by supplying an initial state, before you
+    ever compose the 'Producer':
+
+> import Pipes.Lift
+>
+> naturals :: Monad m => Producer Int m r
+> naturals = evalStateP 0 numbers
+
+    This deletes 'StateT' from the base monad entirely, give you a completely
+    pure 'Pipes.Producer':
+
+>>> Pipes.Prelude.toList naturals
+[0,1,2,3,4,5,6...]
+
+    Note that the convention for the 'S.StateT' run functions is backwards from
+    @transformers@ for convenience: the initial state is the first argument.
+
+    All of these functions internally use 'distribute', which can pull out most
+    monad transformers from the base monad.  For example, 'evalStateP' is
+    defined in terms of 'distribute':
+
+> evalStateP s p = evalStateT (distribute p) s
+
+    Therefore you can use 'distribute' to run other monad transformers, too, as
+    long as they implement the 'MFunctor' type class from the @mmorph@ library.
+-}
