@@ -80,6 +80,9 @@ module Pipes.Tutorial (
 
     -- * Appendix: Types
     -- $types
+
+    -- * Appendix: Time Complexity
+    -- $timecomplexity
     ) where
 
 import Control.Category
@@ -238,9 +241,9 @@ import Prelude hiding ((.), id)
     also an 'Effect':
 
 @
- data 'Void'  -- The uninhabited type
+ data 'X'  -- The uninhabited type
 
-\ type 'Effect' m r = 'Producer' 'Void' m r
+\ type 'Effect' m r = 'Producer' 'X' m r
 @
 
     This is why 'for' permits two different type signatures.  The first type
@@ -249,10 +252,10 @@ import Prelude hiding ((.), id)
 @
  'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Producer' b    m ()) -> 'Producer' b    m r
 
-\ -- Specialize \'b\' to \'Void\'
- 'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Producer' 'Void' m ()) -> 'Producer' 'Void' m r
+\ -- Specialize \'b\' to \'X\'
+ 'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Producer' 'X' m ()) -> 'Producer' 'X' m r
 
-\ -- Producer Void = Effect
+\ -- Producer X = Effect
  'for' :: 'Monad' m => 'Producer' a m r -> (a -> 'Effect'        m ()) -> 'Effect'        m r
 @
 
@@ -341,7 +344,7 @@ import Prelude hiding ((.), id)
     You can also use 'for' to loop over lists, too.  To do so, convert the list
     to a 'Producer' using 'each', which is exported by default from "Pipes":
 
-> each :: (Monad m) => [a] -> Producer a m ()
+> each :: Monad m => [a] -> Producer a m ()
 > each as = mapM_ yield as
 
     Combine 'for' and 'each' to iterate over lists using a \"foreach\" loop:
@@ -375,7 +378,7 @@ import Prelude hiding ((.), id)
 > import Pipes
 > import qualified Pipes.Prelude as P  -- Pipes.Prelude already has 'stdinLn'
 > 
-> duplicate :: (Monad m) => a -> Producer a m ()
+> duplicate :: Monad m => a -> Producer a m ()
 > duplicate x = do
 >     yield x
 >     yield x
@@ -423,9 +426,9 @@ import Prelude hiding ((.), id)
     equality, which always holds no matter what:
 
 @
- \-\- s :: (Monad m) =>      'Producer' a m ()  -- i.e. \'P.stdinLn\'
- \-\- f :: (Monad m) => a -> 'Producer' b m ()  -- i.e. \'duplicate\'
- \-\- g :: (Monad m) => b -> 'Producer' c m ()  -- i.e. \'(lift . putStrLn)\'
+ \-\- s :: Monad m =>      'Producer' a m ()  -- i.e. \'P.stdinLn\'
+ \-\- f :: Monad m => a -> 'Producer' b m ()  -- i.e. \'duplicate\'
+ \-\- g :: Monad m => b -> 'Producer' c m ()  -- i.e. \'(lift . putStrLn)\'
 
 \ for (for s f) g = for s (\\x -> for (f x) g)
 @
@@ -434,7 +437,7 @@ import Prelude hiding ((.), id)
     following operator that is the point-free counterpart to 'for':
 
 @
- (~>) :: (Monad m)
+ (~>) :: Monad m
       => (a -> 'Producer' b m r)
       -> (b -> 'Producer' c m r)
       -> (a -> 'Producer' c m r)
@@ -445,9 +448,9 @@ import Prelude hiding ((.), id)
     into the following more symmetric equation:
 
 @
- f :: (Monad m) => a -> 'Producer' b m r
- g :: (Monad m) => b -> 'Producer' c m r
- h :: (Monad m) => c -> 'Producer' d m r
+ f :: Monad m => a -> 'Producer' b m r
+ g :: Monad m => b -> 'Producer' c m r
+ h :: Monad m => c -> 'Producer' d m r
 
 \ \-\- Associativity
  (f ~> g) ~> h = f ~> (g ~> h)
@@ -605,7 +608,7 @@ ABC
     following intermediate 'Consumer' that requests two 'String's and returns
     them concatenated:
 
-> doubleUp :: (Monad m) => Consumer String m String
+> doubleUp :: Monad m => Consumer String m String
 > doubleUp = do
 >     str1 <- await
 >     str2 <- await
@@ -809,7 +812,7 @@ You shall not pass!
     quirks.  In fact, we can continue the analogy to Unix by defining 'cat'
     (named after the Unix @cat@ utility), which reforwards elements endlessly:
 
-> cat :: (Monad m) => Pipe a a m r
+> cat :: Monad m => Pipe a a m r
 > cat = forever $ do
 >     x <- await
 >     yield x
@@ -835,10 +838,10 @@ You shall not pass!
 > import qualified Pipes.Prelude as P  -- Pipes.Prelude provides 'take', too
 > import Prelude hiding (head)
 >
-> head :: (Monad m) => Int -> Pipe a a m ()
+> head :: Monad m => Int -> Pipe a a m ()
 > head = P.take
 >
-> yes :: (Monad m) => Producer String m r
+> yes :: Monad m => Producer String m r
 > yes = forever $ yield "y"
 >
 > main = runEffect $ yes >-> head 3 >-> P.stdoutLn
@@ -973,7 +976,7 @@ quit<Enter>
     For example, you can loop over the output of a 'Pipe' using 'for', which is
     how 'P.map' is defined:
 
-> map :: (Monad m) => (a -> b) -> Pipe a b m r
+> map :: Monad m => (a -> b) -> Pipe a b m r
 > map f = for cat $ \x -> yield (f x)
 >
 > -- Read this as: For all values flowing downstream, apply 'f'
@@ -987,7 +990,7 @@ quit<Enter>
     You can also feed a 'Pipe' input using ('>~').  This means we could have
     instead defined the @yes@ pipe like this:
 
-> yes :: (Monad m) => Producer String m r
+> yes :: Monad m => Producer String m r
 > yes = return "y" >~ cat
 >
 > -- Read this as: Keep feeding "y" downstream
@@ -999,7 +1002,7 @@ quit<Enter>
     You can also sequence two 'Pipe's together.  This is how 'P.drop' is
     defined:
 
-> drop :: (Monad m) => Int -> Pipe a a m r
+> drop :: Monad m => Int -> Pipe a a m r
 > drop n = do
 >     replicateM_ n await
 >     cat
@@ -1033,7 +1036,7 @@ quit<Enter>
     Another neat thing to know is that 'every' has a more general type:
 
 @
- 'every' :: ('Enumerable' t) => t m a -> 'Producer' a m ()
+ 'every' :: ('Monad' m, 'Enumerable' t) => t m a -> 'Producer' a m ()
 @
 
     'Enumerable' generalizes 'Foldable' and if you have an effectful container
@@ -1041,7 +1044,7 @@ quit<Enter>
     container implement the 'toListT' method of the 'Enumerable' class:
 
 > class Enumerable t where
->     toListT :: (Monad m) => t m a -> ListT m a
+>     toListT :: Monad m => t m a -> ListT m a
 
     You can even use 'Enumerable' to traverse effectful types that are not even
     proper containers, like 'Control.Monad.Trans.Maybe.MaybeT':
@@ -1153,49 +1156,49 @@ Fail<Enter>
     * Polymorphic type synonyms that don't explicitly close unused inputs or
       outputs
 
-    The concrete type synonyms use @()@ to close unused inputs and 'Void' (the
+    The concrete type synonyms use @()@ to close unused inputs and 'X' (the
     uninhabited type) to close unused outputs:
 
     * 'Effect': explicitly closes both ends, forbidding 'await's and 'yield's
 
-> type Effect = Proxy Void () () Void 
+> type Effect = Proxy X () () X
 >
->    Upstream | Downstream
->        +---------+
->        |         |
-> Void  <==       <== ()
->        |         |
-> ()    ==>       ==> Void
->        |    |    |
->        +----|----+
->             v
->             r
+>  Upstream | Downstream
+>     +---------+
+>     |         |
+> X  <==       <== ()
+>     |         |
+> () ==>       ==> X
+>     |    |    |
+>     +----|----+
+>          v
+>          r
 
     * 'Producer': explicitly closes the upstream end, forbidding 'await's
 
-> type Producer b = Proxy Void () () b
+> type Producer b = Proxy X () () b
 >
->    Upstream | Downstream
->        +---------+
->        |         |
-> Void  <==       <== ()
->        |         |
-> ()    ==>       ==> b
->        |    |    |
->        +----|----+
->             v
->             r
+> Upstream | Downstream
+>     +---------+
+>     |         |
+> X  <==       <== ()
+>     |         |
+> () ==>       ==> b
+>     |    |    |
+>     +----|----+
+>          v
+>          r
 
     * 'Consumer': explicitly closes the downstream end, forbidding 'yield's
 
-> type Consumer a = Proxy () a () Void
+> type Consumer a = Proxy () a () X
 >
 > Upstream | Downstream
 >     +---------+
 >     |         |
 > () <==       <== ()
 >     |         |
-> a  ==>       ==> Void
+> a  ==>       ==> X
 >     |    |    |
 >     +----|----+
 >          v
@@ -1221,30 +1224,30 @@ Fail<Enter>
     'Producer', 'Pipe', and a 'Consumer', you can think of information flowing
     like this:
 
->           Producer                Pipe                 Consumer
->        +-----------+          +----------+          +------------+
->        |           |          |          |          |            |
-> Void  <==         <==   ()   <==        <==   ()   <==          <== ()
->        |  stdinLn  |          |  take 3  |          |  stdoutLn  |
-> ()    ==>         ==> String ==>        ==> String ==>          ==> Void
->        |     |     |          |    |     |          |      |     |
->        +-----|-----+          +----|-----+          +------|-----+
->              v                     v                       v
->              ()                    ()                      ()
+>        Producer                Pipe                 Consumer
+>     +-----------+          +----------+          +------------+
+>     |           |          |          |          |            |
+> X  <==         <==   ()   <==        <==   ()   <==          <== ()
+>     |  stdinLn  |          |  take 3  |          |  stdoutLn  |
+> () ==>         ==> String ==>        ==> String ==>          ==> X
+>     |     |     |          |    |     |          |      |     |
+>     +-----|-----+          +----|-----+          +------|-----+
+>           v                     v                       v
+>           ()                    ()                      ()
 
      Composition fuses away the intermediate interfaces, leaving behind an
      'Effect':
 
->                       Effect
->        +-----------------------------------+
->        |                                   |
-> Void  <==                                 <== ()
->        |  stdinLn >-> take 3 >-> stdoutLn  |
-> ()    ==>                                 ==> Void
->        |                                   |
->        +----------------|------------------+
->                         v
->                         ()
+>                    Effect
+>     +-----------------------------------+
+>     |                                   |
+> X  <==                                 <== ()
+>     |  stdinLn >-> take 3 >-> stdoutLn  |
+> () ==>                                 ==> X
+>     |                                   |
+>     +----------------|------------------+
+>                      v
+>                      ()
 
     @pipes@ also provides polymorphic type synonyms with apostrophes at the end
     of their names.  These use universal quantification to leave open any unused
@@ -1358,27 +1361,46 @@ Fail<Enter>
 
     However, polymorphic type synonyms cause problems in many other cases:
 
-    * They induce higher-rank types and require you to enable the @RankNTypes@
-      extension to use them in your own type signatures.
-
-    * They give the wrong behavior when used in the negative position of a
-      function like this:
+    * They usually give the wrong behavior when used as the argument of a
+      function (known as the \"negative\" or \"contravariant\" position) like
+      this:
 
 > f :: Producer' a m r -> ...  -- Wrong
 >
 > f :: Producer  a m r -> ...  -- Right
 
-    * You can't use them within other types without the @ImpredicativeTypes@
-      extension:
+      The former function only accepts polymorphic 'Producer's as arguments.
+      The latter function accepts both polymorphic and concrete 'Producer's,
+      which is probably what you want.
 
-> io :: IO (Producer' a m r)  -- Type error
+    * Even when you desire a polymorphic argument, this induces a higher-ranked
+      type, because it translates to a @forall@ which you cannot factor out to
+      the top-level to simplify the type signature:
 
-    * You can't partially apply them:
+> f :: (forall x' x y' . Proxy x' x y' m r) -> ...
+
+      These kinds of type signatures require the @RankNTypes@ extension.
+
+    * Even when you have polymorphic type synonyms as the result of a function
+      (i.e.  the \"positive\" or \"covariant\" position), recent versions of
+      @ghc@ such still require the @RankNTypes@ extension.  For example, the
+      'Pipes.Prelude.fromHandle' function from "Pipes.Prelude" requires
+      @RankNTypes@ to compile correctly on @ghc-7.6.3@:
+
+> fromHandle :: MonadIO m => Handle -> Producer' String m ()
+
+    * You can't use polymorphic type synonyms inside other type constructors
+      without the @ImpredicativeTypes@ extension:
+
+> io :: IO (Producer' a m r)  -- Type error without ImpredicativeTypes
+
+    * You can't partially apply polymorphic type synonyms:
 
 > stack :: MaybeT (Producer' a m) r  -- Type error
 
     In these scenarios you should fall back on the concrete type synonyms, which
-    are better behaved.
+    are better behaved.  If concrete type synonyms are unsatisfactory, then ask
+    @ghc@ to infer the most general type signature and use that.
 
     For the purposes of debugging type errors you can just remember that:
 
@@ -1395,26 +1417,26 @@ Fail<Enter>
 
 >>> runEffect P.stdinLn
 <interactive>:4:5:
-    Couldn't match expected type `Void' with actual type `String'
+    Couldn't match expected type `X' with actual type `String'
     Expected type: Effect m0 r0
-      Actual type: Proxy Void () () String IO ()
+      Actual type: Proxy X () () String IO ()
     In the first argument of `runEffect', namely `P.stdinLn'
     In the expression: runEffect P.stdinLn
 
     'runEffect' expects an 'Effect', which is equivalent to the following type:
 
-> Effect          IO () = Proxy Void () () Void   IO ()
+> Effect          IO () = Proxy X () () X      IO ()
 
     ... but 'P.stdinLn' type-checks as a 'Producer', which has the following
     type:
 
-> Producer String IO () = Proxy Void () () String IO ()
+> Producer String IO () = Proxy X () () String IO ()
 
     The fourth type variable (the output) does not match.  For an 'Effect' this
-    type variable should be closed (i.e. 'Void'), but 'P.stdinLn' has a 'String'
+    type variable should be closed (i.e. 'X'), but 'P.stdinLn' has a 'String'
     output, thus the type error:
 
->    Couldn't match expected type `Void' with actual type `String'
+>    Couldn't match expected type `X' with actual type `String'
 
     Any time you get type errors like these you can work through them by
     expanding out the type synonyms and seeing which type variables do not
@@ -1423,13 +1445,13 @@ Fail<Enter>
     You may also consult this table of type synonyms to more easily compare
     them:
 
-> type Effect             = Proxy Void () () Void
-> type Producer         b = Proxy Void () () b
-> type Consumer    a      = Proxy ()   a  () Void
-> type Pipe        a    b = Proxy ()   a  () b
+> type Effect             = Proxy X  () () X
+> type Producer         b = Proxy X  () () b
+> type Consumer    a      = Proxy () a  () X
+> type Pipe        a    b = Proxy () a  () b
 >
-> type Server        b' b = Proxy Void () b' b 
-> type Client   a' a      = Proxy a'   a  () Void
+> type Server        b' b = Proxy X  () b' b 
+> type Client   a' a      = Proxy a' a  () X
 >
 > type Effect'            m r = forall x' x y' y . Proxy x' x y' y m r
 > type Producer'        b m r = forall x' x      . Proxy x' x () b m r
@@ -1439,3 +1461,48 @@ Fail<Enter>
 > type Client'  a' a      m r = forall      y' y . Proxy a' a y' y m r
 
 -}
+
+{- $timecomplexity
+    There are three functions that give quadratic time complexity when used in
+    within @pipes@:
+
+    * 'sequence'
+
+    * 'replicateM'
+
+    * 'mapM'
+
+    For example, the time complexity of this code segment scales quadratically
+    with `n`:
+
+> import Control.Monad (replicateM)
+> import Pipes
+>
+> quadratic :: Int -> Consumer a m [a]
+> quadratic n = replicateM n await
+
+    These three functions are generally bad practice to use, because all three
+    of them correspond to \"ListT done wrong\", building a list in memory
+    instead of streaming results.
+
+    However, sometimes situations arise where one deliberately intends to build
+    a list in memory.  The solution is to use the \"codensity transformation\"
+    to transform the code to run with linear time complexity.  This involves:
+
+    * wrapping the code in the @Codensity@ monad transformer (from
+      @Control.Monad.Codensity@ module of the @kan-extensions@ package) using
+      'lift'
+
+    * applying 'sequence' \/ 'replicateM' \/ 'mapM'
+
+    * unwrapping the code using @lowerCodensity@
+
+    To illustrate this, we'd transform the above example to:
+
+> import Control.Monad.Codensity (lowerCodensity)
+> 
+> linear :: Monad m => Int -> Consumer a m [a]
+> linear n = lowerCodensity $ replicateM n $ lift await
+
+    This will produce the exact same result, but in linear time.
+-}
